// Create closure to keep namespace clean and hide implementation.
(function($) {
  var NL_FOURPP_REGEX = /[0-9]{4,4}\s?/;
  var NL_SIXPP_REGEX = /[0-9]{4,4}\s?[a-zA-Z]{2,2}/;

  $.fn.applyLocator = function(options) {
    var instance = this;

    function getConfig(field, default_value) {
      if (typeof options === 'undefined' ||
        typeof options[field] === 'undefined') {
        return default_value;
      } else {
        // Developer chose to specify form field manually.
        return options[field];
      }
    }

    instance.postcode = getConfig('postcode', instance.find('.postcode'));
    instance.target_nl_fourpps = getConfig('target_nl_fourpps', instance.find('.target_nl_fourpps'));

    instance.message = getConfig('message', instance.find('.message'));
    instance.spinner = getConfig('spinner', instance.find('.spinner'));
    instance.output = getConfig('output', instance.find('.output'));
    instance.per_page = getConfig('per_page', 4);

    instance.city = getConfig('city', instance.find('.city'));
    instance.distance = getConfig('distance', instance.find('.distance'));
    instance.lat = getConfig('lat', instance.find('.lat'));
    instance.lng = getConfig('lng', instance.find('.lng'));
    instance.nl_fourpp = getConfig('nl_fourpp', instance.find('.nl_fourpp'));

    instance.assemble_func = getConfig('assemble_func', assemble_one);
    instance.callback = getConfig('callback');

    // Locator kicks into action by button submit, or (default) automatically.
    var submit_buttons = this.find(':submit');
    if (submit_buttons.length >= 1) {
      submit_buttons.click(function() {
        locator(instance);
        return false;
      });
    } else {
      instance.postcode.keyup(function() {
        locator(instance);
      });
    }
  };

  var pro6pp_cache = {};
  function pro6pp_cached_get(obj, url, params, callback) {
    var key = url + $.param(params);
    if (pro6pp_cache.hasOwnProperty(key)) {
      if (typeof callback !== 'undefined') {
        callback(obj, pro6pp_cache[key]);
      }
    } else {
      $.getJSON(url + "?callback=?", params, function(data) {
        pro6pp_cache[key] = data;
        if (typeof callback !== 'undefined') {
          callback(obj, data);
        }
      });
    }
  }

  function locator(obj) {
    obj.message.hide().empty();
    var postcode = obj.postcode.val();
    var target_nl_fourpps = obj.target_nl_fourpps.val();
    // Trigger on '5408'
    if (NL_SIXPP_REGEX.test(postcode) || NL_FOURPP_REGEX.test(postcode)) {
      var url = 'https://api.pro6pp.nl/v1/locator';
      var params = new Object();
      params.auth_key = pro6pp_auth_key;
      params.per_page = obj.per_page;
      if (NL_SIXPP_REGEX.test(postcode)) {
        params.nl_sixpp = postcode;
      } else {
        params.nl_fourpp = postcode;
      }
      params.target_nl_fourpps = target_nl_fourpps;
      pro6pp_cached_get(obj, url, params, fillin);
    }
  }

  function assemble_one(result) {
    distance_km = Math.round(result.distance / 1000);
    return "Winkel gevonden (" + result.nl_fourpp + ") op een afstand van " + distance_km + " km<br />";
  }

  function fillin(obj, json) {
    if (typeof obj.callback !== 'undefined') {
      obj.callback(json.results);
    }

    if (json.status == 'ok') {
      obj.output.empty();

      $.each(json.results, function(i, result) {
        obj.output.append(obj.assemble_func(result)).show();
      });
      if (json.results.length > 0) {
        obj.city.val(json.results[0].city);
        obj.distance.val(json.results[0].distance);
        obj.nl_fourpp.val(json.results[0].nl_fourpp);
        obj.lat.val(json.results[0].lat);
        obj.lng.val(json.results[0].lng);
      }
    } else {
      var translated_message = json.error.message;
      if (json.error.message == 'nl_fourpp not found') {
        translated_message = 'Onbekende postcode';
      } else if (json.error.message == 'Invalid nl_fourpp format') {
          translated_message = 'Ongeldig postcode formaat';
      } else if (json.error.message == 'invalid target_nl_fourpps format') {
        translated_message = 'Ongeldig target_nl_fourpps formaat';
      }

      obj.message.html(translated_message).show();
    }
  }
})(jQuery);
